// Program version
var versionNum = '2.1.3';

// Default values:
var defaultProvince = '02';
var defaultLocation = '111123'; // 서울 종로구 효제동
var defaultAirDataIsHidden = false; // Hidden status of detailed air quality data

// Real values:
var myProvince, myLocation, airDataIsHidden;

// URLs for air quality data updates
var airQualityURL = 'http://www.airkorea.or.kr/realSearch24?';

var airKoreaIframe, updateTimerID, animateTimerID, versionRequest;
var lastUpdated = null;

// Regular Expressions for data extracting
var airDataIndexColorRegEx = /\/images\/main\/area_point_(\d)\.png/g;
var airDataValueRegEx = /<td>(\d+(\.\d+)?|-)<\/td>/g;
var airDataHasNoValueRegEx = /\/images\/main\/area_point_7\.png/;

// for the localization
function getLocalizedString(string) {
  try { string = localizedStrings[string] || string; } catch (e) {}
  return string;
}

// Create unique key for each instance of widget
function createkey(key) {
  return widget.identifier + '-' + key;
}

window.onload = setup;

function setup() {
  var doc = document;
  airKoreaIframe = document.getElementById('airKoreaIframe');
  airKoreaIframe.addEventListener('load', updateAirData, false);
  doc.getElementById('air-quality').addEventListener('click', getAirData, false);
  doc.getElementById('slider').addEventListener('click', showHideDetail, false);
  doc.getElementById('newVersion').addEventListener('click', goHome, false);
  doc.getElementById('linkGoHome').addEventListener('click', goHome, false);
  doc.getElementById('air-korea').addEventListener('click', goAirKorea, false);
  doc.getElementById('info').addEventListener('click', showBack, false);
  doc.getElementById('done').addEventListener('click', showFront, false);
  var doneString = getLocalizedString('Done');
  CreateGlassButton('done', { text: doneString, onclick: 'showFront' });
  CreateInfoButton('info', { 'backgroundStyle': 'black', 'creationFunction': 'CreateInfoButton', 'foregroundStyle': 'white', 'frontID': 'front', 'onclick': 'showBack' });

  // Try to retrieve preferences, if they exist
  if (window.widget) {
    myProvince = widget.preferenceForKey(createkey('province'));
    myLocation = widget.preferenceForKey(createkey('location'));
    airDataIsHidden = widget.preferenceForKey(createkey('airdata'));
  }

  if (myProvince === undefined) {
    myProvince = defaultProvince;
    widget.setPreferenceForKey(myProvince, createkey('province'));
  }

  if (myLocation === undefined) {
    myLocation = defaultLocation;
    widget.setPreferenceForKey(myLocation, createkey('location'));
  }

  if (airDataIsHidden === undefined) {
    airDataIsHidden = defaultAirDataIsHidden;
    widget.setPreferenceForKey(airDataIsHidden, createkey('airdata'));
  }
  var airDataContainer = document.getElementById('air-quality-data-container');
  if (airDataIsHidden) {
    airDataContainer.classList.add('hidden');
  } else {
    airDataContainer.classList.remove('hidden');
  }

  populateProvinceSelect(myProvince);

  doc.getElementById('pm10-label').title = getLocalizedString('Particulate Matter 10');
  doc.getElementById('pm25-label').title = getLocalizedString('Particulate Matter 2.5');
  doc.getElementById('o-label').title = getLocalizedString('Ozone');
  doc.getElementById('no-label').title = getLocalizedString('Nitrogen dioxide');
  doc.getElementById('co-label').title = getLocalizedString('Carbon monoxide');
  doc.getElementById('so-label').title = getLocalizedString('Sulfur dioxide');
  doc.getElementById('province-label').innerText = getLocalizedString('Province');
  doc.getElementById('location-label').innerText = getLocalizedString('Location');
  doc.getElementById('remarks-label').innerText = getLocalizedString('CAI Remarks');
  doc.getElementById('good-label').innerText = getLocalizedString('Good');
  doc.getElementById('moderate-label').innerText = getLocalizedString('Moderate');
  doc.getElementById('unhealthy-label').innerText = getLocalizedString('Unhealthy');
  doc.getElementById('very-unhealthy-label').innerText = getLocalizedString('Very Unhealthy');
  doc.getElementById('nodata-label').innerText = getLocalizedString('No Data');
  doc.getElementById('version-label').innerText = getLocalizedString('Version') + versionNum;

  getAirData();
  addObservationSiteName();
  checkForNewVersion();
}

function getAirData() {
  airKoreaIframe.src = airQualityURL + 'tm_x='  + stationCoordinates[myLocation].tm_x + '&tm_y=' + stationCoordinates[myLocation].tm_y + '&' + Date.now();
  document.getElementById('air-quality').classList.add('updating');
}

function updateAirData() {
  updateTimerID = window.setTimeout(function () {
    var responseData = airKoreaIframe.contentDocument.querySelector('#realTable tbody').innerHTML;
    var i, j, startIndex, noOfData, firstRawHasNoData, currentData = [], currentIndexColor = [], currentValue = [], doc = document;

    currentData = responseData.match(airDataIndexColorRegEx);
    if (currentData) {
      firstRawHasNoData = airDataHasNoValueRegEx.test(currentData[0]);
      if (firstRawHasNoData) {
        startIndex = 7;
      } else {
        startIndex = 0;
        lastUpdated = Date.now();
      }
      noOfData = 7;
      for (i = startIndex, j = startIndex + noOfData; i < j; i++) {
        currentIndexColor[i - startIndex] = getIndexColor(currentData[i].replace(airDataIndexColorRegEx, '$1'));
      }

      currentData = responseData.match(airDataValueRegEx);
      startIndex = firstRawHasNoData ? 10 : 0;
      noOfData = 9;
      for (i = startIndex, j = startIndex + noOfData; i < j; i++) {
        currentValue[i - startIndex] = currentData[i].replace(airDataValueRegEx, '$1');
      }

      var CAIValue = currentValue[0];
      doc.getElementById('cai-bar').style.width = CAIValue / 500 * 100 +'%';
      doc.getElementById('cai-value').innerText = 'CAI: ' + CAIValue + ' (/500)';
      changeCAIBarColor(currentIndexColor[0]);
      doc.getElementById('pm10-index').style.color = currentIndexColor[1];
      doc.getElementById('pm10-value').innerText = currentValue[1] + ' ㎍/㎥';
      doc.getElementById('pm25-index').style.color = currentIndexColor[2];
      doc.getElementById('pm25-value').innerText = currentValue[2] + ' ㎍/㎥';
      doc.getElementById('o-index').style.color = currentIndexColor[3];
      doc.getElementById('o-value').innerText = currentValue[3] + ' ppm';
      doc.getElementById('no-index').style.color = currentIndexColor[4];
      doc.getElementById('no-value').innerText = currentValue[4] + ' ppm';
      doc.getElementById('co-index').style.color = currentIndexColor[5];
      doc.getElementById('co-value').innerText = currentValue[5] + ' ppm';
      doc.getElementById('so-index').style.color = currentIndexColor[6];
      doc.getElementById('so-value').innerText = currentValue[6] + ' ppm';
    } else { // just after the midnight, data table is empty about an hour.
      doc.getElementById('cai-bar').style.width = 0;
      doc.getElementById('cai-value').innerText = 'CAI: - (/500)';
      doc.getElementById('pm10-index').style.color = '#999';
      doc.getElementById('pm10-value').innerText = '-';
      doc.getElementById('pm25-index').style.color = '#999';
      doc.getElementById('pm25-value').innerText = '-';
      doc.getElementById('o-index').style.color = '#999';
      doc.getElementById('o-value').innerText = '-';
      doc.getElementById('no-index').style.color = '#999';
      doc.getElementById('no-value').innerText = '-';
      doc.getElementById('co-index').style.color = '#999';
      doc.getElementById('co-value').innerText = '-';
      doc.getElementById('so-index').style.color = '#999';
      doc.getElementById('so-value').innerText = '-';
    }

    emphasizeMainPollutant(firstRawHasNoData);

    animateTimerID = window.setTimeout(function() {
      document.getElementById('air-quality').classList.remove('updating');
    }, 300);
  }, 900);
}

var mainPollutantId;
function emphasizeMainPollutant(firstRawHasNoData) {
  var rawData, mainPollutant;

  if (mainPollutantId) {
    document.getElementById(mainPollutantId).classList.remove('main-pollutant');
  }

  if (firstRawHasNoData) {
    rawData = airKoreaIframe.contentDocument.querySelector('#realTable tbody > tr:nth-child(2)');
  } else {
    rawData = airKoreaIframe.contentDocument.querySelector('#realTable tbody > tr:nth-child(1)');
  }
  mainPollutant = rawData.querySelector('td:nth-of-type(2)').textContent;

  switch (mainPollutant) {
    case 'PM10':
      mainPollutantId = 'pm10-index';
      break;
    case 'PM2.5':
      mainPollutantId = 'pm25-index';
      break;
    case '오존':
      mainPollutantId = 'o-index';
      break;
    case '이산화질소':
      mainPollutantId = 'no-index';
      break;
    case '일산화탄소':
      mainPollutantId = 'co-index';
      break;
    case '아황산가스':
      mainPollutantId = 'so-index';
      break;
    default:
      mainPollutantId = '';
  }

  if (mainPollutantId !== '') {
    document.getElementById(mainPollutantId).classList.add('main-pollutant');
  }
}

function addObservationSiteName() {
  var locationSelect = document.getElementById('location-popup');
  var siteName = locationSelect[locationSelect.selectedIndex].text;
  siteName = siteName.split(' ');
  if (siteName.length == 2) {
    siteName = siteName[1];
  } else {
    siteName = siteName[1] + ' ' + siteName[2];
  }
  document.getElementById('cai-value').title = getLocalizedString('CAI') + ' ▸ ' + siteName;
}

function getIndexColor(indexColor) {
  switch (indexColor) {
    case '1':
      return '#0000FF';
    case '2':
      return '#00FF00';
    case '3':
      return '#FFFF00';
    case '4':
    case '5':
      return '#FF0000';
    case '0': // no data
    case '6':
    case '7':
      return '#999';
  }
}

function changeCAIBarColor(indexColor) {
  var CAIBar = document.getElementById('air-quality');
  CAIBar.className = 'updating';
  switch (indexColor) {
    case '#0000FF':
      CAIBar.classList.add('blue');
      break;
    case '#00FF00':
      CAIBar.classList.add('green');
      break;
    case '#FFFF00':
      CAIBar.classList.add('yellow');
      break;
    case '#FF0000':
      CAIBar.classList.add('red');
      break;
  }
}

function populateProvinceSelect(province) {
  var provinceSelect = document.getElementById('province-popup');
  var oFragment = document.createDocumentFragment();
  var doc = document;

  for (var i = 0, j = localizedProvinceNames.length; i < j; ++i) {
    var element = doc.createElement('option');
    element.text = localizedProvinceNames[i].name;
    element.value = localizedProvinceNames[i].code;
    if (localizedProvinceNames[i].code == province) {
      element.selected = true;
    }
    oFragment.appendChild(element);
  }
  provinceSelect.appendChild(oFragment);

  provinceSelect.addEventListener('change', provincechanged, false);
  populateLocationSelect();
}

function provincechanged() {
  var provinceSelect = document.getElementById('province-popup');
  var locationSelect = document.getElementById('location-popup');
  var provinceValue = provinceSelect.options[provinceSelect.selectedIndex].value;
  if (myProvince != provinceValue) {
    var nodelist = locationSelect.childNodes;
    var numOfList = nodelist.length, this_node;
    for (var i = numOfList-1; i > -1; --i) {
      this_node = nodelist[i];
      locationSelect.removeChild(this_node);
    }
    myProvince = provinceValue;
    widget.setPreferenceForKey(myProvince, createkey('province'));

    populateLocationSelect();
  }
}

function populateLocationSelect() {
  var provinceSelect = document.getElementById('province-popup');
  var locationSelect = document.getElementById('location-popup');
  var provinceValue = provinceSelect.options[provinceSelect.selectedIndex].value;
  var i, j;

  if (locationSelect.hasChildNodes()) {
    var nodelist = locationSelect.childNodes;
    var numOfList = nodelist.length, this_node;
    for (i = numOfList-1; i > -1; --i) {
      this_node = nodelist[i];
      locationSelect.removeChild(this_node);
    }
  }

  var oFragment = document.createDocumentFragment();
  var locationList = localizedLocationNames[provinceValue];
  locationList.sort(sortLocationName);

  var doc = document;
  for (i = 0, j = locationList.length; i < j; ++i) {
    var element = doc.createElement('option');
    element.text = locationList[i].name;
    element.value = locationList[i].code;
    if (locationList[i].code == myLocation) {
      element.selected = true;
    }
    oFragment.appendChild(element);
  }
  locationSelect.appendChild(oFragment);
  if (locationSelect.selectedIndex == -1) {
    locationSelect.selectedIndex = 0;
  }

  locationSelect.addEventListener('change', locationchanged, false);
  locationchanged();
}

function locationchanged() {
  var locationSelect = document.getElementById('location-popup');
  var selectedLocation = locationSelect.options[locationSelect.selectedIndex].value;
  if (selectedLocation != myLocation) {
    myLocation = selectedLocation;
    widget.setPreferenceForKey(myLocation, createkey('location'));
  }
  addObservationSiteName();
  getAirData();
}

function sortLocationName(a,b) {
  if (a.name > b.name) return 1;
  else if ( a.name < b.name) return -1;
  else return 0;
}

function showHideDetail() {
  var airDataContainer = document.getElementById('air-quality-data-container');
  if (airDataIsHidden) {
    airDataContainer.classList.remove('hidden');
    airDataIsHidden = false;
  } else {
    airDataContainer.classList.add('hidden');
    airDataIsHidden = true;
  }
  widget.setPreferenceForKey(airDataIsHidden, createkey('airdata'));
}

function remove() {
  widget.setPreferenceForKey(null, createkey('province'));
  widget.setPreferenceForKey(null, createkey('location'));
  widget.setPreferenceForKey(null, createkey('airdata'));
}

function hide() {
  window.clearTimeout(updateTimerID);
  window.clearTimeout(animateTimerID);
  document.getElementById('air-quality').classList.remove('updating');
}

function show() {
  // Air data updates every hours
  var timeNow = Date.now();
  // If air data has been updated over 10 mins ago
  if (timeNow - lastUpdated >= 1000 * 60 * 10) {
    getAirData();
  }
}

if (window.widget) {
  widget.onremove = remove;
  widget.onhide = hide;
  widget.onshow = show;
}

function showBack() {
  document.querySelector('.flip-container').classList.add('flipped');
}

function showFront() {
  document.querySelector('.flip-container').classList.remove('flipped');
}

function goHome() {
  widget.openURL('https://appletree.or.kr/widgets/AirOutside/');
}

function goAirKorea() {
  widget.openURL('http://www.airkorea.or.kr');
}

// Download version.txt from website and check version numbers
function checkForNewVersion() {
  var versionURL = 'https://appletree.or.kr/widgets/AirOutside/version.txt';

  if (versionRequest) {
    versionRequest.abort();
  }

  versionRequest = new XMLHttpRequest();
  versionRequest.onreadystatechange = getVersionNumber;
  versionRequest.open('GET', versionURL, true);
  versionRequest.send(null);
}

// If new version is available, display link and notice
function getVersionNumber() {
  try {
    if (versionRequest.readyState == 4 && versionRequest.status == 200) {
      if (versionRequest.responseText !== versionNum) {
        document.getElementById('newMessage').innerText = getLocalizedString('NewVersion');
        document.getElementById('newVersion').style.display = 'block';
      }

      versionRequest = null;
    } else if (versionRequest.readyState == 4 && versionRequest.status != 200) {
      versionRequest.abort();
      versionRequest = null;
    }
  }
  catch(exception) {
    //drawCheckFailure(getLocalizedString('Failed') + exception);
  }
}
